library reflector.mirrors;

import 'dart:mirrors';

import 'package:reflector/src/interfaces.dart';

class MirrorsReflector implements Reflector {
  factory MirrorsReflector.fromLibraries(Iterable<Symbol> libraries) {
    return new MirrorsReflector._(libraries.map((library) {
      return currentMirrorSystem().findLibrary(library).declarations.values;
    }).expand((i) => i));
  }

  factory MirrorsReflector.fromTypes(Iterable<Type> types) {
    return new MirrorsReflector._(types.map((type) => reflectClass(type)));
  }

  final Iterable<DeclarationMirror> _mirrors;

  MirrorsReflector._(this._mirrors);

  @override
  List<Metadata> reflect([Options options = const Options()]) {
    return _mirrors.where((mirror) {
      if (mirror is! ClassMirror) return false;
      if (options.withAnnotations.isNotEmpty) {
        final annotations = mirror.metadata.map((m) {
          return m.type.reflectedType;
        }).toList(growable: false);
        if (!options.withAnnotations.every(annotations.contains)) {
          return false;
        }
      }
      return true;
    }).map((DeclarationMirror mirror) {
      if (mirror is ClassMirror) {
        final annotations = mirror.metadata.map((m) {
          return new AnnotationInfo(m.type.simpleName, m.type.reflectedType);
        }).toList(growable: false);
        return new ClassInfo(mirror.simpleName, annotations: annotations);
      } else {
        throw new UnsupportedError('Invalid type: ${mirror.runtimeType}.');
      }
    }).toList(growable: false);
  }
}
