library reflector.source;

import 'dart:mirrors';

import 'package:analyzer/analyzer.dart';

import 'package:reflector/src/interfaces.dart';

part 'src/source/visitor.dart';

/// Implementation of [Reflector] that uses reads Dart source code.
class SourceReflector implements Reflector {
  final CompilationUnit _unit;

  /// Create a reflector from existing Dart [sourceCode].
  factory SourceReflector.fromCode(String sourceCode) {
    return new SourceReflector._(parseCompilationUnit(sourceCode));
  }

  /// Create a reflector from an existing file at [filePath] of Dart source.
  factory SourceReflector.fromFile(String filePath) {
    return new SourceReflector._(parseDartFile(filePath));
  }

  SourceReflector._(this._unit);

  /// Reflects over [Metadata] that matches the criteria in [options].
  @override
  List<Metadata> reflect([Options options = const Options()]) {
    final visitor = new _ReflectingVisitor(options)
      ..visitCompilationUnit(_unit);
    return visitor.results;
  }
}
