library reflector.src.interfaces;

import 'dart:collection';

/// Generic interface for metadata reflection.
abstract class Reflector {
  /// Returns a collection of metadata matching [options].
  Iterable<Metadata> reflect([Options options = const Options()]);
}

/// Options configuration for [Reflector.reflect].
class Options {
  final List<Type> withAnnotations;

  const Options({this.withAnnotations: const []});
}

/// Base class for reflected metadata.
class Metadata {
  final Symbol name;

  Metadata(this.name);
}

/// Annotation information.
class AnnotationInfo extends Metadata {
  final Type type;

  AnnotationInfo(Symbol name, this.type) : super(name);
}

/// Class of information for metadata of a defined class [Type].
class ClassInfo extends Metadata {
  /// Annotations expressed on the class definition.
  final List<AnnotationInfo> annotations;

  /// Create a new [ClassInfo] with [annotations].
  factory ClassInfo(Symbol name, {Iterable<AnnotationInfo> annotations}) {
    if (annotations != null) {
      annotations = annotations.toList(growable: false);
      annotations = new UnmodifiableListView<AnnotationInfo>(annotations);
    } else {
      annotations = const [];
    }

    return new ClassInfo._(name, annotations);
  }

  ClassInfo._(Symbol name, this.annotations) : super(name);
}
