library reflector.testing.common;

import 'package:guinness/guinness.dart';
import 'package:matcher/matcher.dart' hide expect;

import 'package:reflector/reflector.dart';
import 'package:reflector/testing/annotations.dart';

void runTests(Reflector getReflector(), {bool isRuntime: false}) {
  it('returns a named metadata class for every class', () {
    final metadata = getReflector().reflect().toList();
    expect(metadata).to(hasLength(3));
    expect(metadata.map((m) => m.name).toList()).to([#A, #B, #C]);
  });

  it('returns only classes annotated with "@Annotation"', () {
    final metadata = getReflector().reflect(const Options(
      withAnnotations: const [Annotation]
    )).toList();
    expect(metadata).to(hasLength(1));
    expect(metadata.first.name, #A);
  });

  it('returns a metadata class with metadata annotations attached', () {
    final ClassInfo metadata = getReflector().reflect().first;
    expect(metadata.annotations).to(hasLength(1));
    expect(metadata.annotations.first.name).toEqual(#Annotation);
    if (isRuntime) {
      expect(metadata.annotations.first.type).toEqual(Annotation);
    }
  });
}
